@@ -9,7 +9,7 @@ package HTML::Parser;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '3.1990';  # $Date: 2001/03/13 19:37:42 $
+$VERSION = '3.1991';  # $Date: 2001/03/19 19:19:19 $
 
 require HTML::Entities;
 
@@ -111,11 +111,9 @@ sub parse_file
 sub netscape_buggy_comment  # legacy
 {
     my $self = shift;
-    if ($^W) {
-	require Carp;
-	Carp::carp("netscape_buggy_comment() is deprecated.  " .
-	    "Please use the strict_comment() method instead");
-    }
+    require Carp;
+    Carp::carp("netscape_buggy_comment() is deprecated.  " .
+	       "Please use the strict_comment() method instead");
     my $old = !$self->strict_comment;
     $self->strict_comment(!shift) if @_;
     return $old;
@@ -387,6 +385,11 @@ enabled, blocks of text are always reported in one piece.  This will
 delay the text event until the following (non-text) event has been
 recognized by the parser.
 
+Note that the C<offset> argspec will give you the offset of the first
+segment of text and C<length> is the combined length of the segments.
+Since there might be ignored tags in between, these numbers can't be
+used to directly index in the original document file.
+
 =item $p->marked_sections( [$bool] )
 
 By default, section markings like <![CDATA[...]]> are treated like
@@ -491,7 +494,7 @@ suppressed.
 
 =item $p->report_only_tags( TAG, ... )
 
-Any C<start> and C<end> events I<not> involving any of the tags given
+Any C<start> and C<end> events involving any of the tags I<not> given
 are suppressed.
 
 =item $p->ignore_elements( TAG, ... )
